import React, { useEffect, useState, useCallback, useRef } from 'react';
import { 
  Container, 
  List, 
  ListItem, 
  ListItemText,
  TextField,
  Button,
  IconButton, 
  Typography,
  Avatar,
  ListItemAvatar,
} from '@mui/material';
import { Delete as DeleteIcon, Edit as EditIcon } from '@mui/icons-material';
import axios from 'axios';
import { useParams, useLocation } from 'react-router-dom'; 
import ItemDetails from './ItemDetails';


export default function Items({ token }) {
  const { id: boxId } = useParams(); 
  const [items, setItems] = useState([]);
  const [newItemName, setNewItemName] = useState('');
  const [newItemDescription, setNewItemDescription] = useState('');
  const [newItemImagePath, setNewItemImagePath] = useState('/images/default.jpg');
  const [editingItem, setEditingItem] = useState(null);
  const location = useLocation();
  const boxName = location.state?.boxName || 'Unknown Box';
  const [itemImages, setItemImages] = useState({});
  const fileInputRef = useRef(null);


  // const handleSelectItem = (item) => {
  //   setSelectedItem(item);
  // };

  const [selectedItem, setSelectedItem] = React.useState(null);

  const handleCloseItemDetails = () => {
    setEditingItem(null); // Close the ItemDetails modal
  };

  const handleImageError = (e) => {
    if (e.target.src.startsWith('data:image/')) {
      console.error("Default image failed to load. Check the file path.");
      return; 
    }
    const reader = new FileReader();
    reader.onload = () => {
      e.target.onerror = null;
      e.target.src = reader.result;
    };
    fetch('/default.jpg')
      .then(res => res.blob())
      .then(blob => reader.readAsDataURL(blob))
      .catch(error => console.error("Error loading default image:", error));
  };

  const getImageSrc = useCallback((itemId) => {
    return axios.get(`${process.env.REACT_APP_API_URL}/items/${itemId}/image`, {
      headers: { Authorization: `Bearer ${token}` },
      responseType: 'blob'
    })
    .then(response => {
      if (response.status === 200) {
        return new Promise((resolve, reject) => {
          const reader = new FileReader();
          reader.onload = () => resolve(reader.result);
          reader.onerror = reject;
          reader.readAsDataURL(response.data);
        });
      } else {
        throw new Error('Image fetch failed');
      }
    })
    .catch(() => {
      return new Promise((resolve, reject) => {
        const img = new Image();
        img.src = '/default.jpg'; 
        img.onload = () => {
          const canvas = document.createElement('canvas');
          canvas.width = img.width;
          canvas.height = img.height;
          const ctx = canvas.getContext('2d');
          ctx.drawImage(img, 0, 0);
          resolve(canvas.toDataURL());
        };
        img.onerror = reject;
      });
    });
  }, [token]);

  const fetchItems = useCallback(() => {
    axios.get(`${process.env.REACT_APP_API_URL}/boxes/${boxId}/items`, {
        headers: { Authorization: `Bearer ${token}` }
    }).then(response => {
        setItems(response.data);

        // Fetch images for each item
        response.data.forEach(item => {
          getImageSrc(item.ID).then(imageDataUrl => {
            setItemImages(prevItemImages => ({
              ...prevItemImages,
              [item.ID]: imageDataUrl
            }));
          });
        });
    });
  }, [boxId, token, getImageSrc]);

  useEffect(() => {
    fetchItems();
  }, [boxId, token, fetchItems]);

  const handleAddItem = () => {
    const formData = new FormData();
    formData.append('name', newItemName);
    formData.append('description', newItemDescription);
    formData.append('box_id', parseInt(boxId, 10));
    // Append image only if a new one is selected
    if (fileInputRef.current.files[0]) {
      formData.append('image', fileInputRef.current.files[0]);
    }

    axios.post(`${process.env.REACT_APP_API_URL}/items`, formData, {
      headers: { 
        Authorization: `Bearer ${token}`,
        'Content-Type': 'multipart/form-data' // Important for file uploads
      }
    }).then(() => {
      setNewItemName('');
      setNewItemDescription('');
      setNewItemImagePath('');
      // Clear the file input
      if (fileInputRef.current) {
        fileInputRef.current.value = '';
      }
      fetchItems();
    });
  };

  const handleDeleteItem = (itemId) => {
    axios.delete(`${process.env.REACT_APP_API_URL}/items/${itemId}`, {
      headers: { Authorization: `Bearer ${token}` }
    }).then(() => {
      fetchItems();
    });
  };

  const handleEditItem = (item) => {
    setEditingItem(item);
  };

  const handleSaveEdit = () => {
    setEditingItem(null);
    fetchItems();
  };

  return (
    <Container>
      <h2>Items in Box: {boxName}</h2>

      <TextField
        label="Item Name"
        variant="outlined"
        fullWidth
        margin="normal"
        value={newItemName}
        onChange={(e) => setNewItemName(e.target.value)}
      />
      <TextField
        label="Item Description"
        variant="outlined"
        fullWidth
        margin="normal"
        value={newItemDescription}
        onChange={(e) => setNewItemDescription(e.target.value)}
      />
      <TextField 
        label="Item Image Path" 
        variant="outlined" 
        fullWidth 
        margin="normal"
        value={newItemImagePath}
        onChange={(e) => setNewItemImagePath(e.target.files[0].name)}          
      />
      <input 
        type="file" 
        accept="image/*" 
        ref={fileInputRef} 
        style={{ display: 'none' }} 
        id="newItemImageUpload" // Unique ID
      />
      <Button variant="contained" component="label" htmlFor="newItemImageUpload"> 
        Upload Image
        {/* ... rest of your button code ... */}
      </Button>
      <Button variant="contained" color="primary" onClick={handleAddItem}>
        Add Item
      </Button>

      {editingItem ? (
        <ItemDetails 
          item={editingItem} 
          token={token} 
          onSave={handleSaveEdit} 
          onClose={handleCloseItemDetails} 
          boxId={boxId}
          />
      ) : (
        <List>
        {items.map((item) => (
          <ListItem key={item.ID} secondaryAction={
            <>
              <IconButton edge="end" onClick={() => handleEditItem(item)}>
                <EditIcon />
              </IconButton>
              <IconButton edge="end" onClick={() => handleDeleteItem(item.ID)}>
                <DeleteIcon />
              </IconButton>
            </>
          }>
            <ListItemAvatar> 
              <Avatar
                src={itemImages[item.ID]} 
                alt={item.name}
                onError={handleImageError} 
              />
            </ListItemAvatar>
            <ListItemText 
              primary={item.name} 
              secondary={
                <>
                  <Typography variant="body2">{item.description}</Typography>
                  {item.image_path && (
                    <Typography variant="caption">Image: {item.image_path}</Typography>
                  )}
                </>
              } 
            />
          </ListItem>
        ))}
        </List>
      )}
    </Container>
  );
}